#define EQU .equ
#define DB  .db
#define DW  .dw
#define DS  .ds

;  module pt3player
  
;Vortex Tracker II v1.0 PT3 player for ZX Spectrum
;(c)2004,2007 S.V.Bulba <vorobey@mail.khstu.ru>
;http://bulba.untergrund.net (http://bulba.at.kz)

;Release number
Release EQU 037H

;Features
;--------
;-Can be compiled at any address (i.e. no need rounding ORG
; address).
;-Variables (VARS) can be located at any address (not only after
;code block).
;-INIT subroutine detects module version and rightly generates
; both note and volume tables outside of code block (in VARS).
;-Two portamento (spc. command 3xxx) algorithms (depending of
; module version).
;-New 1.XX and 2.XX special command behaviour (only for PT v3.7
; and higher).
;-Any Tempo value are accepted (including Tempo=1 and Tempo=2).
;-Fully compatible with Ay_Emul PT3 player codes.
;-See also notes at the end of this source code.

;Limitations
;-----------
;-Can run in RAM only (self-modified code is used).

;Warning!!! PLAY subroutine can crash if no module are loaded
;into RAM or INIT subroutine was not called before.

;Call MUTE or INIT one more time to mute sound after stopping
;playing 

;  .ORG 0C000h

;Test codes (commented)
; CALL START
; EI
;_LP  HALT
; CALL START+5
; XOR A
; IN A,(#FE)
; CPL
; AND 15
; JR Z,_LP
; JR START+8

TonA  EQU 0
TonB  EQU 2
TonC  EQU 4
Noise EQU 6
Mixer EQU 7
AmplA EQU 8
AmplB EQU 9
AmplC EQU 10
Env EQU 11
EnvTp EQU 13

FS_RST EQU 0f6CDh
SD_INIT EQU 0f3f7h
SD_ON EQU 0f397h
FS_FNDF EQU 0f81eh
FS_FNDN EQU 0f84ah
PRINT_NEWLINE EQU 0f349h
PRINTN EQU 0f083h
PRHEX EQU 0f25dh
F_OPEN EQU 0F8FDh
F_READ EQU 0F919h
PUTC EQU 0F24Fh

    .ORG 2C18h
    DB  10h, 10h, 0, 0, 0, "GalaxyMusicPT3", 0, 0, 0C7h, 10h, 0, 0, 0A5h
    DB  36h, 2Ch, 035h, 3Ch
    DB  3Ah, 2Ch, 049h, 2Ch, 1, 0
    DB  "A=USR(&2C50)", 0Dh, 0, 0, 0, 0, 0, 0, 0
    .ORG 2C50h
; ---------------------------------------------------------------------------
STRT:
  LD HL,PT3_cnt
  LD (HL),0
  LD HL,Identifier
  LD B,22
  CALL PRINTN

;  CALL SD_INIT
  CALL SD_ON
;  CALL FS_RST
  ld de,PT3_mask
  CALL FS_FNDF
  JR C,C_NO_FILE
FL_LOOP:
  PUSH HL
  LD B,8
  CALL PRINTN
  LD C,' '
  CALL PUTC
  LD B,3
  CALL PRINTN
  LD C,' '
  CALL PUTC
  POP HL
  PUSH HL
  LD A,L
  add A,01Dh
  LD L,A
  LD A,(HL)
  LD D,A
  call PRHEX
  dec HL
  LD A,(HL)
  LD E,A
  call PRHEX
  CALL PRINT_NEWLINE
  POP HL
;CALL FS_FNDF ; DE - name of file for finding, out HL - string FAT record of file
;CALL F_OPEN ; DE - name of file for opening, out C==1 if file not founded
;CALL F_READ ; HL - addr for store readed data, BC - size of readed data
  EX DE,HL
  push HL
  call F_NAME_CPY
  LD DE,MDLADDR
  CALL F_OPEN
  POP BC

  LD HL,MDLADDR
  CALL F_READ
  LD HL,PT3_cnt
  INC (HL)
  LD HL,PT3_tmp
  LD (HL),0
  CALL PLAY_PT3 ;play PT3 !

  ld de,PT3_mask
  CALL FS_FNDF
FL_LOOP2:
  CALL FS_FNDN
  JR NC,FL_LOOP1
C_NO_FILE:
  ret ;exit

FL_LOOP1:
  push HL
  LD HL,PT3_tmp
  INC (HL)
  LD A,(HL)
  LD HL,PT3_cnt
  sub (HL)
  pop HL
  jr z,FL_LOOP
  jr FL_LOOP2

PLAY_PT3:
    ld  a, 1
    ld  (SETUP), a
    call  START
    ei
loc_2C59:       ; CODE XREF: ROM:2C6Dj
;  push HL
;  LD HL,CrPsPtr
;  LD A,(HL)
;  call PRHEX
;  LD HL,02a68h  ;cursor position
;  LD A,(HL)
;  SUB 02h
;  LD (HL),A
;  pop HL
    ld  a, (201Fh) ; if press <space> return
    and 1
    jp  nz, loc_2C65
    call  sub_2C78
    ret

F_NAME_CPY:
  LD HL,MDLADDR
  LD c,8
nam_cpy:
  LD A,(DE)
  LD (HL),A
  inc de
  inc hl
  dec c
  jr nz,nam_cpy
  LD A,'.'
  LD (HL),A
  inc HL
  LD c,3
ext_cpy:
  LD A,(DE)
  LD (HL),A
  inc de
  inc hl
  dec c
  jr nz,ext_cpy
  LD A,0
  LD (HL),A
  ret

; ---------------------------------------------------------------------------

loc_2C65:       ; CODE XREF: ROM:2C5Ej
    halt
    call  sub_2C75
    ld  a, (SETUP)
    rla
    jr  nc, loc_2C59
    ret

;Entry and other points
;START initialization
;START+3 initialization with module address in HL
;START+5 play one quark
;START+8 mute
;START+10 setup and status flags
;START+11 pointer to current position value in PT3 module;
;After INIT (START+11) points to Postion0-1 (optimization)

START:
 LD HL,MDLADDR
 JR INIT
sub_2C75:
 JP PLAY
sub_2C78:
 JR MUTE
SETUP DB 0 ;set bit0 to 1, if you want to play without looping
       ;bit7 is set each time, when loop point is passed
CrPsPtr DW 0 

PT3_cnt: DB 0
PT3_tmp: DB 0

PT3_mask: DB "*.PT3",0

Identifier:
  DB "=VTII PT3 PLAYER R.",Release,"=",0dh

CHECKLP: LD HL,SETUP
  SET 7,(HL)
  BIT 0,(HL)
  RET Z
  POP HL
  LD HL,DelyCnt
  INC (HL)
  LD HL,ChanA+NtSkCn
  INC (HL)
MUTE:  XOR A
  LD H,A
  LD L,A
  LD (AYREGS+AmplA),A
  LD (AYREGS+AmplB),HL
  JP ROUT_A0

INIT:
;HL - AddressOfModule

  LD (MODADDR),HL
  LD (MDADDR2),HL
  PUSH HL
  LD DE,100
  ADD HL,DE
  LD A,(HL)
  LD (Delay),A
  PUSH HL
  POP IX
  ADD HL,DE
  LD (CrPsPtr),HL
  LD E,(IX+102-100)
  ADD HL,DE
  INC HL
  LD (LPosPtr),HL
  POP DE
  LD L,(IX+103-100)
  LD H,(IX+104-100)
  ADD HL,DE
  LD (PatsPtr),HL
  LD HL,169
  ADD HL,DE
  LD (OrnPtrs),HL
  LD HL,105
  ADD HL,DE
  LD (SamPtrs),HL
  LD HL,SETUP
  RES 7,(HL)

;note table data depacker
  LD DE,T_PACK
  LD BC,T1_+(2*49)-1
TP_0  LD A,(DE)
  INC DE
  CP 15*2
  JR NC,TP_1
  LD H,A
  LD A,(DE)
  LD L,A
  INC DE
  JR TP_2
TP_1  PUSH DE
  LD D,0
  LD E,A
  ADD HL,DE
  ADD HL,DE
  POP DE
TP_2  LD A,H
  LD (BC),A
  DEC BC
  LD A,L
  LD (BC),A
  DEC BC
  SUB 0ffh&(0F8h*2)
  JR NZ,TP_0

  LD HL,VARS
  LD (HL),A
  LD DE,VARS+1
  LD BC,VAR0END-VARS-1
  LDIR
  INC A
  LD (DelyCnt),A
  LD HL,0F001h ;H - CHP.Volume, L - CHP.NtSkCn
  LD (ChanA+NtSkCn),HL
  LD (ChanB+NtSkCn),HL
  LD (ChanC+NtSkCn),HL

  LD HL,EMPTYSAMORN
  LD (AdInPtA),HL ;ptr to zero
  LD (ChanA+OrnPtr),HL ;ornament 0 is "0,1,0"
  LD (ChanB+OrnPtr),HL ;in all versions from
  LD (ChanC+OrnPtr),HL ;3.xx to 3.6x and VTII

  LD (ChanA+SamPtr),HL ;S1 There is no default
  LD (ChanB+SamPtr),HL ;S2 sample in PT3, so, you
  LD (ChanC+SamPtr),HL ;S3 can comment S1,2,3; see
            ;also EMPTYSAMORN comment

  LD A,(IX+13-100) ;EXTRACT VERSION NUMBER
  SUB 30h
  JR C,L20
  CP 10
  JR C,L21
L20 LD A,6
L21 LD (Version),A
  PUSH AF
  CP 4
  LD A,(IX+99-100) ;TONE TABLE NUMBER
  RLA
  AND 7

;NoteTableCreator (c) Ivan Roshin
;A - NoteTableNumber*2+VersionForNoteTable
;(xx1b - 3.xx..3.4r, xx0b - 3.4x..3.6x..VTII1.0)

  LD HL,NT_DATA
  PUSH DE
  LD D,B
  ADD A,A
  LD E,A
  ADD HL,DE
  LD E,(HL)
  INC HL
  SRL E
  SBC A,A
  AND 0A7h ;#00 (NOP) or #A7 (AND A)
  LD (L3),A
  EX DE,HL
  POP BC ;BC=T1_
  ADD HL,BC

  LD A,(DE)
  ADD A,T_&255
  LD C,A
  ADC A,T_/256
  SUB C
  LD B,A
  PUSH BC
  LD DE,NT_
  PUSH DE

  LD B,12
L1  PUSH BC
  LD C,(HL)
  INC HL
  PUSH HL
  LD B,(HL)

  PUSH DE
  EX DE,HL
  LD DE,23
  .db 0DDh,026h,08h ;  LD IXH,8 :?????????????

L2  SRL B
  RR C
L3  DB 19h  ;AND A or NOP
  LD A,C
  ADC A,D ;=ADC 0
  LD (HL),A
  INC HL
  LD A,B
  ADC A,D
  LD (HL),A
  ADD HL,DE
  .db 0DDh,25h ;  DEC IXH  ;????????????????
  JR NZ,L2

  POP DE
  INC DE
  INC DE
  POP HL
  INC HL
  POP BC
  DJNZ L1

  POP HL
  POP DE

  LD A,E
  CP TCOLD_1&255
  JR NZ,CORR_1
  LD A,0FDh
  LD (NT_+2Eh),A

CORR_1  LD A,(DE)
  AND A
  JR Z,TC_EXIT
  RRA
  PUSH AF
  ADD A,A
  LD C,A
  ADD HL,BC
  POP AF
  JR NC,CORR_2
  DEC (HL)
  DEC (HL)
CORR_2  INC (HL)
  AND A
  SBC HL,BC
  INC DE
  JR CORR_1

TC_EXIT

  POP AF

;VolTableCreator (c) Ivan Roshin
;A - VersionForVolumeTable (0..4 - 3.xx..3.4x;
         ;5.. - 3.5x..3.6x..VTII1.0)

  CP 5
  LD HL,11h
  LD D,H
  LD E,H
  LD A,17h
  JR NC,M1
  DEC L
  LD E,L
  XOR A
M1      LD (M2),A

  LD IX,VT_+16
  LD C,10h

INITV2  PUSH HL

  ADD HL,DE
  EX DE,HL
  SBC HL,HL

INITV1  LD A,L
M2      DB 7Dh
  LD A,H
  ADC A,0
  LD (IX),A
  INC IX
  ADD HL,DE
  INC C
  LD A,C
  AND 15
  JR NZ,INITV1

  POP HL
  LD A,E
  CP 77h
  JR NZ,M3
  INC E
M3      LD A,C
  AND A
  JR NZ,INITV2

  JP ROUT_A0

;pattern decoder
PD_OrSm LD (IX-12+Env_En),0
  CALL SETORN
  LD A,(BC)
  INC BC
  RRCA

PD_SAM  ADD A,A
PD_SAM_ LD E,A
  LD D,0
SamPtrs EQU $+1
  LD HL,2121h
  ADD HL,DE
  LD E,(HL)
  INC HL
  LD D,(HL)
MODADDR EQU $+1
  LD HL,2121h
  ADD HL,DE
  LD (IX-12+SamPtr),L
  LD (IX-12+SamPtr+1),H
  JR PD_LOOP

PD_VOL  RLCA
  RLCA
  RLCA
  RLCA
  LD (IX-12+Volume),A
  JR PD_LP2
  
PD_EOff LD (IX-12+Env_En),A
  LD (IX-12+PsInOr),A
  JR PD_LP2

PD_SorE DEC A
  JR NZ,PD_ENV
  LD A,(BC)
  INC BC
  LD (IX-12+NNtSkp),A
  JR PD_LP2

PD_ENV  CALL SETENV
  JR PD_LP2

PD_ORN  CALL SETORN
  JR PD_LOOP

PD_ESAM LD (IX-12+Env_En),A
  LD (IX-12+PsInOr),A
  CALL NZ,SETENV
  LD A,(BC)
  INC BC
  JR PD_SAM_

PTDECOD LD A,(IX-12+Note)
  LD (PrNote+1),A
  LD L,(IX-12+CrTnSl)
  LD H,(IX-12+CrTnSl+1)
  LD (PrSlide+1),HL

PD_LOOP LD DE,2010h
PD_LP2  LD A,(BC)
  INC BC
  ADD A,E
  JR C,PD_OrSm
  ADD A,D
  JR Z,PD_FIN
  JR C,PD_SAM
  ADD A,E
  JR Z,PD_REL
  JR C,PD_VOL
  ADD A,E
  JR Z,PD_EOff
  JR C,PD_SorE
  ADD A,96
  JR C,PD_NOTE
  ADD A,E
  JR C,PD_ORN
  ADD A,D
  JR C,PD_NOIS
  ADD A,E
  JR C,PD_ESAM
  ADD A,A
  LD E,A
  LD HL,0ffffh&(SPCCOMS+0FF20h-2000h)
  ADD HL,DE
  LD E,(HL)
  INC HL
  LD D,(HL)
  PUSH DE
  JR PD_LOOP

PD_NOIS LD (Ns_Base),A
  JR PD_LP2

PD_REL  RES 0,(IX-12+Flags)
  JR PD_RES
  
PD_NOTE LD (IX-12+Note),A
  SET 0,(IX-12+Flags)
  XOR A

PD_RES  LD (PDSP_+1),SP
  LD SP,IX
  LD H,A
  LD L,A
  PUSH HL
  PUSH HL
  PUSH HL
  PUSH HL
  PUSH HL
  PUSH HL
PDSP_ LD SP,3131h

PD_FIN  LD A,(IX-12+NNtSkp)
  LD (IX-12+NtSkCn),A
  RET

C_PORTM RES 2,(IX-12+Flags)
  LD A,(BC)
  INC BC
;SKIP PRECALCULATED TONE DELTA (BECAUSE
;CANNOT BE RIGHT AFTER PT3 COMPILATION)
  INC BC
  INC BC
  LD (IX-12+TnSlDl),A
  LD (IX-12+TSlCnt),A
  LD DE,NT_
  LD A,(IX-12+Note)
  LD (IX-12+SlToNt),A
  ADD A,A
  LD L,A
  LD H,0
  ADD HL,DE
  LD A,(HL)
  INC HL
  LD H,(HL)
  LD L,A
  PUSH HL
PrNote  LD A,3Eh
  LD (IX-12+Note),A
  ADD A,A
  LD L,A
  LD H,0
  ADD HL,DE
  LD E,(HL)
  INC HL
  LD D,(HL)
  POP HL
  SBC HL,DE
  LD (IX-12+TnDelt),L
  LD (IX-12+TnDelt+1),H
  LD E,(IX-12+CrTnSl)
  LD D,(IX-12+CrTnSl+1)
Version EQU $+1
  LD A,3Eh
  CP 6
  JR C,OLDPRTM ;Old 3xxx for PT v3.5-
PrSlide LD DE,1111h
  LD (IX-12+CrTnSl),E
  LD (IX-12+CrTnSl+1),D
OLDPRTM LD A,(BC) ;SIGNED TONE STEP
  INC BC
  EX AF,AF'
  LD A,(BC)
  INC BC
  AND A
  JR Z,NOSIG
  EX DE,HL
NOSIG SBC HL,DE
  JP P,SET_STP
  CPL
  EX AF,AF'
  NEG
  EX AF,AF'
SET_STP LD (IX-12+TSlStp+1),A
  EX AF,AF'
  LD (IX-12+TSlStp),A
  LD (IX-12+COnOff),0
  RET

C_GLISS SET 2,(IX-12+Flags)
  LD A,(BC)
  INC BC
  LD (IX-12+TnSlDl),A
  AND A
  JR NZ,GL36
  LD A,(Version) ;AlCo PT3.7+
  CP 7
  SBC A,A
  INC A
GL36  LD (IX-12+TSlCnt),A
  LD A,(BC)
  INC BC
  EX AF,AF'
  LD A,(BC)
  INC BC
  JR SET_STP

C_SMPOS LD A,(BC)
  INC BC
  LD (IX-12+PsInSm),A
  RET

C_ORPOS LD A,(BC)
  INC BC
  LD (IX-12+PsInOr),A
  RET

C_VIBRT LD A,(BC)
  INC BC
  LD (IX-12+OnOffD),A
  LD (IX-12+COnOff),A
  LD A,(BC)
  INC BC
  LD (IX-12+OffOnD),A
  XOR A
  LD (IX-12+TSlCnt),A
  LD (IX-12+CrTnSl),A
  LD (IX-12+CrTnSl+1),A
  RET

C_ENGLS LD A,(BC)
  INC BC
  LD (Env_Del),A
  LD (CurEDel),A
  LD A,(BC)
  INC BC
  LD L,A
  LD A,(BC)
  INC BC
  LD H,A
  LD (ESldAdd),HL
  RET

C_DELAY LD A,(BC)
  INC BC
  LD (Delay),A
  RET
  
SETENV  LD (IX-12+Env_En),E
  LD (AYREGS+EnvTp),A
  LD A,(BC)
  INC BC
  LD H,A
  LD A,(BC)
  INC BC
  LD L,A
  LD (EnvBase),HL
  XOR A
  LD (IX-12+PsInOr),A
  LD (CurEDel),A
  LD H,A
  LD L,A
  LD (CurESld),HL
C_NOP RET

SETORN  ADD A,A
  LD E,A
  LD D,0
  LD (IX-12+PsInOr),D
OrnPtrs EQU $+1
  LD HL,2121h
  ADD HL,DE
  LD E,(HL)
  INC HL
  LD D,(HL)
MDADDR2 EQU $+1
  LD HL,2121h
  ADD HL,DE
  LD (IX-12+OrnPtr),L
  LD (IX-12+OrnPtr+1),H
  RET

;ALL 16 ADDRESSES TO PROTECT FROM BROKEN PT3 MODULES
SPCCOMS DW C_NOP
  DW C_GLISS
  DW C_PORTM
  DW C_SMPOS
  DW C_ORPOS
  DW C_VIBRT
  DW C_NOP
  DW C_NOP
  DW C_ENGLS
  DW C_DELAY
  DW C_NOP
  DW C_NOP
  DW C_NOP
  DW C_NOP
  DW C_NOP
  DW C_NOP

CHREGS  XOR A
  LD (Ampl),A
  BIT 0,(IX+Flags)
  PUSH HL
  JP Z,CH_EXIT
  LD (CSP_+1),SP
  LD L,(IX+OrnPtr)
  LD H,(IX+OrnPtr+1)
  LD SP,HL
  POP DE
  LD H,A
  LD A,(IX+PsInOr)
  LD L,A
  ADD HL,SP
  INC A
  CP D
  JR C,CH_ORPS
  LD A,E
CH_ORPS LD (IX+PsInOr),A
  LD A,(IX+Note)
  ADD A,(HL)
  JP P,CH_NTP
  XOR A
CH_NTP  CP 96
  JR C,CH_NOK
  LD A,95
CH_NOK  ADD A,A
  EX AF,AF'
  LD L,(IX+SamPtr)
  LD H,(IX+SamPtr+1)
  LD SP,HL
  POP DE
  LD H,0
  LD A,(IX+PsInSm)
  LD B,A
  ADD A,A
  ADD A,A
  LD L,A
  ADD HL,SP
  LD SP,HL
  LD A,B
  INC A
  CP D
  JR C,CH_SMPS
  LD A,E
CH_SMPS LD (IX+PsInSm),A
  POP BC
  POP HL
  LD E,(IX+TnAcc)
  LD D,(IX+TnAcc+1)
  ADD HL,DE
  BIT 6,B
  JR Z,CH_NOAC
  LD (IX+TnAcc),L
  LD (IX+TnAcc+1),H
CH_NOAC EX DE,HL
  EX AF,AF'
  LD L,A
  LD H,0
  LD SP,NT_
  ADD HL,SP
  LD SP,HL
  POP HL
  ADD HL,DE
  LD E,(IX+CrTnSl)
  LD D,(IX+CrTnSl+1)
  ADD HL,DE
CSP_  LD SP,3131h
  EX (SP),HL
  XOR A
  OR (IX+TSlCnt)
  JR Z,CH_AMP
  DEC (IX+TSlCnt)
  JR NZ,CH_AMP
  LD A,(IX+TnSlDl)
  LD (IX+TSlCnt),A
  LD L,(IX+TSlStp)
  LD H,(IX+TSlStp+1)
  LD A,H
  ADD HL,DE
  LD (IX+CrTnSl),L
  LD (IX+CrTnSl+1),H
  BIT 2,(IX+Flags)
  JR NZ,CH_AMP
  LD E,(IX+TnDelt)
  LD D,(IX+TnDelt+1)
  AND A
  JR Z,CH_STPP
  EX DE,HL
CH_STPP SBC HL,DE
  JP M,CH_AMP
  LD A,(IX+SlToNt)
  LD (IX+Note),A
  XOR A
  LD (IX+TSlCnt),A
  LD (IX+CrTnSl),A
  LD (IX+CrTnSl+1),A
CH_AMP  LD A,(IX+CrAmSl)
  BIT 7,C
  JR Z,CH_NOAM
  BIT 6,C
  JR Z,CH_AMIN
  CP 15
  JR Z,CH_NOAM
  INC A
  JR CH_SVAM
CH_AMIN CP -15
  JR Z,CH_NOAM
  DEC A
CH_SVAM LD (IX+CrAmSl),A
CH_NOAM LD L,A
  LD A,B
  AND 15
  ADD A,L
  JP P,CH_APOS
  XOR A
CH_APOS CP 16
  JR C,CH_VOL
  LD A,15
CH_VOL  OR (IX+Volume)
  LD L,A
  LD H,0
  LD DE,VT_
  ADD HL,DE
  LD A,(HL)
CH_ENV  BIT 0,C
  JR NZ,CH_NOEN
  OR (IX+Env_En)
CH_NOEN LD (Ampl),A
  BIT 7,B
  LD A,C
  JR Z,NO_ENSL
  RLA
  RLA
  SRA A
  SRA A
  SRA A
  ADD A,(IX+CrEnSl) ;SEE COMMENT BELOW
  BIT 5,B
  JR Z,NO_ENAC
  LD (IX+CrEnSl),A
NO_ENAC LD HL,AddToEn
  ADD A,(HL) ;BUG IN PT3 - NEED WORD HERE.
       ;FIX IT IN NEXT VERSION?
  LD (HL),A
  JR CH_MIX
NO_ENSL RRA
  ADD A,(IX+CrNsSl)
  LD (AddToNs),A
  BIT 5,B
  JR Z,CH_MIX
  LD (IX+CrNsSl),A
CH_MIX  LD A,B
  RRA
  AND 48h
CH_EXIT LD HL,AYREGS+Mixer
  OR (HL)
  RRCA
  LD (HL),A
  POP HL
  XOR A
  OR (IX+COnOff)
  RET Z
  DEC (IX+COnOff)
  RET NZ
  XOR (IX+Flags)
  LD (IX+Flags),A
  RRA
  LD A,(IX+OnOffD)
  JR C,CH_ONDL
  LD A,(IX+OffOnD)
CH_ONDL LD (IX+COnOff),A
  RET

PLAY    XOR A
  LD (AddToEn),A
  LD (AYREGS+Mixer),A
  DEC A
  LD (AYREGS+EnvTp),A
  LD HL,DelyCnt
  DEC (HL)
  JR NZ,PL2
  LD HL,ChanA+NtSkCn
  DEC (HL)
  JR NZ,PL1B
AdInPtA EQU $+1
  LD BC,0101h
  LD A,(BC)
  AND A
  JR NZ,PL1A
  LD D,A
  LD (Ns_Base),A
  LD HL,(CrPsPtr)
  INC HL
  LD A,(HL)
  INC A
  JR NZ,PLNLP
  CALL CHECKLP
LPosPtr EQU $+1
  LD HL,2121h
  LD A,(HL)
  INC A
PLNLP LD (CrPsPtr),HL
  DEC A
  ADD A,A
  LD E,A
  RL D
PatsPtr EQU $+1
  LD HL,2121h
  ADD HL,DE
  LD DE,(MODADDR)
  LD (PSP_+1),SP
  LD SP,HL
  POP HL
  ADD HL,DE
  LD B,H
  LD C,L
  POP HL
  ADD HL,DE
  LD (AdInPtB),HL
  POP HL
  ADD HL,DE
  LD (AdInPtC),HL
PSP_  LD SP,3131h
PL1A  LD IX,ChanA+12
  CALL PTDECOD
  LD (AdInPtA),BC

PL1B  LD HL,ChanB+NtSkCn
  DEC (HL)
  JR NZ,PL1C
  LD IX,ChanB+12
AdInPtB EQU $+1
  LD BC,0101h
  CALL PTDECOD
  LD (AdInPtB),BC

PL1C  LD HL,ChanC+NtSkCn
  DEC (HL)
  JR NZ,PL1D
  LD IX,ChanC+12
AdInPtC EQU $+1
  LD BC,0101h
  CALL PTDECOD
  LD (AdInPtC),BC

Delay EQU $+1
PL1D  LD A,3Eh
  LD (DelyCnt),A

PL2 LD IX,ChanA
  LD HL,(AYREGS+TonA)
  CALL CHREGS
  LD (AYREGS+TonA),HL
  LD A,(Ampl)
  LD (AYREGS+AmplA),A
  LD IX,ChanB
  LD HL,(AYREGS+TonB)
  CALL CHREGS
  LD (AYREGS+TonB),HL
  LD A,(Ampl)
  LD (AYREGS+AmplB),A
  LD IX,ChanC
  LD HL,(AYREGS+TonC)
  CALL CHREGS
; LD A,(Ampl) ;Ampl = AYREGS+AmplC
; LD (AYREGS+AmplC),A
  LD (AYREGS+TonC),HL

  LD HL,(Ns_Base_AddToNs)
  LD A,H
  ADD A,L
  LD (AYREGS+Noise),A

AddToEn EQU $+1
  LD A,3Eh
  LD E,A
  ADD A,A
  SBC A,A
  LD D,A
  LD HL,(EnvBase)
  ADD HL,DE
  LD DE,(CurESld)
  ADD HL,DE
  LD (AYREGS+Env),HL

  XOR A
  LD HL,CurEDel
  OR (HL)
  JR Z,ROUT_A0
  DEC (HL)
  JR NZ,ROUT
Env_Del EQU $+1
  LD A,3Eh
  LD (HL),A
ESldAdd EQU $+1
  LD HL,2121h
  ADD HL,DE
  LD (CurESld),HL

ROUT:  XOR A
ROUT_A0: ; LD DE,0FFBFh
;  LD BC,0FFFDh
;  LD HL,AYREGS
;LOUT:  OUT (C),A
;  LD B,E
;  OUTI 
;  LD B,D
;  INC A
;  CP 13
;  JR NZ,LOUT
;  OUT (C),A
;  LD A,(HL)
;  AND A
;  RET M
;  LD B,E
;  OUT (C),A
;  RET

  LD C,00h
  LD HL,AYREGS
LOUT  OUT (C),A
  INC C
  OUTI 
  DEC C
  INC A
  CP 13
  JR NZ,LOUT
  OUT (C),A
  LD A,(HL)
  AND A
  RET M
  INC C
  OUT (C),A
  RET

NT_DATA: DB (T_NEW_0-T1_)*2
  DB TCNEW_0-T_
  DB (T_OLD_0-T1_)*2+1
  DB TCOLD_0-T_
  DB (T_NEW_1-T1_)*2+1
  DB TCNEW_1-T_
  DB (T_OLD_1-T1_)*2+1
  DB TCOLD_1-T_
  DB (T_NEW_2-T1_)*2
  DB TCNEW_2-T_
  DB (T_OLD_2-T1_)*2
  DB TCOLD_2-T_
  DB (T_NEW_3-T1_)*2
  DB TCNEW_3-T_
  DB (T_OLD_3-T1_)*2
  DB TCOLD_3-T_

T_

TCOLD_0 DB 00h+1,04h+1,08h+1,0Ah+1,0Ch+1,0Eh+1,12h+1,14h+1
  DB 18h+1,24h+1,3Ch+1,0
TCOLD_1 DB 5Ch+1,0
TCOLD_2 DB 30h+1,36h+1,4Ch+1,52h+1,5Eh+1,70h+1,82h,8Ch,9Ch
  DB 9Eh,0A0h,0A6h,0A8h,0AAh,0ACh,0AEh,0AEh,0
TCNEW_3 DB 56h+1
TCOLD_3 DB 1Eh+1,22h+1,24h+1,28h+1,2Ch+1,2Eh+1,32h+1,0BEh+1,0
TCNEW_0 DB 1Ch+1,20h+1,22h+1,26h+1,2Ah+1,2Ch+1,30h+1,54h+1
  DB 0BCh+1,0BEh+1,0
TCNEW_1 EQU TCOLD_1
TCNEW_2 DB 1Ah+1,20h+1,24h+1,28h+1,2Ah+1,3Ah+1,4Ch+1,5Eh+1
  DB 0BAh+1,0BCh+1,0BEh+1,0

EMPTYSAMORN EQU $-1
  DB 1,0,90h ;delete #90 if you don't need default sample

;first 12 values of tone tables (packed)

T_PACK  DB (06ECh*2)/256,(06ECh*2)&255
  DB (0755h-06ECh)
  DB (07C5h-0755h)
  DB (083Bh-07C5h)
  DB (08B8h-083Bh)
  DB (093Dh-08B8h)
  DB (09CAh-093Dh)
  DB (0A5Fh-09CAh)
  DB (0AFCh-0A5Fh)
  DB (0BA4h-0AFCh)
  DB (0C55h-0BA4h)
  DB (0D10h-0C55h)
  DB (066Dh*2)/256,(066Dh*2)&255
  DB (06CFh-066Dh)
  DB (0737h-06CFh)
  DB (07A4h-0737h)
  DB (0819h-07A4h)
  DB (0894h-0819h)
  DB (0917h-0894h)
  DB (09A1h-0917h)
  DB (0A33h-09A1h)
  DB (0ACFh-0A33h)
  DB (0B73h-0ACFh)
  DB (0C22h-0B73h)
  DB (0CDAh-0C22h)
  DB (0704h*2)/256,(0704h*2)&255
  DB (076Eh-0704h)
  DB (07E0h-076Eh)
  DB (0858h-07E0h)
  DB (08D6h-0858h)
  DB (095Ch-08D6h)
  DB (09ECh-095Ch)
  DB (0A82h-09ECh)
  DB (0B22h-0A82h)
  DB (0BCCh-0B22h)
  DB (0C80h-0BCCh)
  DB (0D3Eh-0C80h)
  DB (07E0h*2)/256,(07E0h*2)&255
  DB (0858h-07E0h)
  DB (08E0h-0858h)
  DB (0960h-08E0h)
  DB (09F0h-0960h)
  DB (0A88h-09F0h)
  DB (0B28h-0A88h)
  DB (0BD8h-0B28h)
  DB (0C80h-0BD8h)
  DB (0D60h-0C80h)
  DB (0E10h-0D60h)
  DB (0EF8h-0E10h)

;vars from here can be stripped
;you can move VARS to any other address

VARS:

;ChannelsVars
;  STRUCT  CHP
;reset group
PsInOr  EQU 0
PsInSm  EQU 1
CrAmSl  EQU 2
CrNsSl  EQU 3
CrEnSl  EQU 4
TSlCnt  EQU 5
CrTnSl  EQU 6
TnAcc EQU 8
COnOff  EQU 10
;reset group

OnOffD  EQU 11

;IX for PTDECOD here (+12)
OffOnD  EQU 12
OrnPtr  EQU 13
SamPtr  EQU 15
NNtSkp  EQU 17
Note  EQU 18
SlToNt  EQU 19
Env_En  EQU 20
Flags EQU 21
 ;Enabled - 0,SimpleGliss - 2
TnSlDl  EQU 22
TSlStp  EQU 23
TnDelt  EQU 25
NtSkCn  EQU 27
Volume  EQU 28
;  ENDS
CHP EQU 29

ChanA DS CHP
ChanB DS CHP
ChanC DS CHP

;GlobalVars
DelyCnt DB 0
CurESld DW 0
CurEDel DB 0
Ns_Base_AddToNs
Ns_Base DB 0
AddToNs DB 0

AYREGS:

VT_ DS 256 ;CreatedVolumeTableAddress

EnvBase EQU VT_+14

T1_ EQU VT_+16 ;Tone tables data depacked here

T_OLD_1 EQU T1_
T_OLD_2 EQU T_OLD_1+24
T_OLD_3 EQU T_OLD_2+24
T_OLD_0 EQU T_OLD_3+2
T_NEW_0 EQU T_OLD_0
T_NEW_1 EQU T_OLD_1
T_NEW_2 EQU T_NEW_0+24
T_NEW_3 EQU T_OLD_3

NT_ DS 192 ;CreatedNoteTableAddress

;local var
Ampl  EQU AYREGS+AmplC

VAR0END EQU VT_+16 ;INIT zeroes from VARS to VAR0END-1

VARSEND EQU $

MDLADDR EQU $

;Release 0 steps:
;11.Sep.2004 - Note tables creator
;12.Sep.2004 - Volume tables creator; INIT subroutine
;13.Sep.2004 - Play counters, position counters
;14.Sep.2004 - Patterns decoder subroutine
;15.Sep.2004 - Resting (no code)
;16.Sep.2004 - CHREGS subroutine; global debugging; 1st stable
;version was born
;17.Sep.2004 - Debugging and optimization. First release!
;Release 1 steps:
;20.Sep.2004 - local vars moved to code (selfmodified code
;smaller and faster)
;22.Sep.2004 - added mute sound entry at START+8; position
;pointer moved to START+11; added setup and status byte at
;START+10 noloop mode and loop passed flags added
;Release 2 steps:
;28.Sep.2004 - Optimization: code around CHREGS's volume and
;vibrato faster now; zeroing PD_RES through stack; Ton and Ampl
;moved from channel vars to global ones; first position selector
;removed from INIT; optimization for packers(Ivan Roshin method)
;Release 3 steps:
;2.Oct.2004 - optimization in INIT and PD_LOOP (thanks to Ivan
;Roshin)
;4.Oct.2004 - load delay from (hl) in INIT (2 bytes shorter)
;5.Oct.2004 - optimization in PD_LOOP (->PD_LP2)
;7.Oct.2004 - swaping some commands for better packing
;Release 4 steps:
;9.Oct.2004 - optimization around LD HL,SPCCOMS (thanks to Ivan
;Roshin); in PTDECOD swapped BC and DE to optimize C_PORTM;
;removed sam and orn len and loop channel vars; CHREGS totally
;optimized
;Release 5 steps:
;11.Oct.2004 - PD_OrSm and C_PORTM optimized; Ivan Roshin's
;volume tables creator algorithm (51 bytes shorter than mine)
;12.Oct.2004 - Ivan Roshin's note tables creator algorithm (74
;bytes shorter than mine)
;Release 6 steps:
;14.Oct.2004 - loop and next position calculations moved to INIT
;15.Oct.2004 - AdInPt moved to code
;19.Oct.2004 - Env_Del moved to code
;20.Oct.2004 - Version PUSH and POP (1 byte shorter, thanks to
;Ivan Roshin)
;22.Oct.2004 - Env_En moved from Flags' bit to byte (shorter and
;faster code)
;25.Oct.2004 - SETENV optimized
;29.Oct.2004 - Optimized around AddToEn (SBC A,A, thanks to Ivan
;Roshin)
;3.Nov.2004 - Note tables data was compressed; with depacker it
;is 9 bytes shorter than uncompressed (thanks to Ivan Roshin)
;4.Nov.2004 - default sample and ornament both are fixed now
;and placed into code block (6 bytes shorter)
;7.Nov.2004 - LD A,(Ns_Base):LD L,A changed to LD HL,(Ns_Base)
;(thanks to Dima Bystrov)
;9.Nov.2004 - Ns_Base and AddToNs are merged to Ns_Base_AddToNs;
;LD A,255 changed to DEC A (at start of PLAY); added ROUT_A0
;12.Nov.2004 - NtSkCn&Volume are merged (8 bytes smaller init);
;LD BC,T1_ changed to PUSH DE...POP BC in note table creator
;19.Dec.2004 - NT_DATA reorganized (6 bytes shorter, thanks to
;Ivan Roshin); C_PORTM and C_GLISS are merged via SET_STP (48
;tacts slower, but 8 bytes smaller, thanks to Ivan Roshin)
;Release 7 steps:
;29.Apr.2007 - SjAsm adaptation; new 1.xx and 2.xx
;interpretation for PT 3.7+.

;Tests in IMMATION TESTER V1.0 by Andy Man/POS (thanks to
;Himik's ZxZ for help):
;Module name/author Min tacts Max tacts Average
;Spleen/Nik-O   1720    9256    5500
;Chuta/Miguel   1720    9496    5500
;Zhara/Macros   4536    8744    5500

;Size:
;Code block #651 bytes
;Variables #21D bytes (can be stripped)
;Size in RAM #651+#21D=#86E (2158) bytes

;Notes:
;Pro Tracker 3.4r can not be detected by header, so PT3.4r tone
;tables really used only for modules of 3.3 and older versions.

;  endmodule
  .END
